//-*-c++-*-
/***************************************************************************
 *   Copyright (C) 2003 by Fred Schaettgen                                 *
 *   kdebluetooth@schaettgen.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "phonecontroller.h"
#include "atcommander.h"
#include "soundserver.h"
#include <kdebug.h>
#include <qregexp.h>
#include <klocale.h>
#include <kglobal.h>
#include <kconfig.h>
#include <kiconloader.h>
#include "vorbisencoder.h"
#include <qdatetime.h>
#include <qtimer.h>
#include <qfile.h>
#include <qdatetime.h>
#include <libkbluetooth/rfcommsocket.h>
#include <libkbluetooth/deviceaddress.h>
#include <kdebug.h>

using namespace KBluetooth;

PhoneController::PhoneController(QObject *owner, const char* name,
    KBluetooth::RfcommSocket* socket) :
    QObject(owner, name)
{
    this->socket = socket;
    atcommander = new AtCommander(this, "atcommander", socket);

    /*connect(socket, SIGNAL(connectionClosed()),
        this, SIGNAL(socketClosed()));
    connect(socket, SIGNAL(error(int)),
        this, SIGNAL(socketError(int)));*/

    vorbisEncoder = NULL;
    state.isIncomingCall = true;

    ringTimeoutTimer = new QTimer(this);
    connect(ringTimeoutTimer, SIGNAL(timeout()),
        this, SLOT(slotRingTimeout()));

    soundServer = new SoundServer(this);
    connect(soundServer, SIGNAL(speechConnectionStatus(bool)),
        this, SLOT(slotSpeechConnectionStatusChanged(bool)));

    this->atcommander = atcommander;
    statusUpdate();
}

void PhoneController::init()
{
    if (!atcommander) return;

    connect(atcommander, SIGNAL(unsolicitedResult(QString)),
        this, SLOT(slotUnsolicitedResult(QString)));

    // Send test command. This will initialize the
    // indicator mapping.
    atcommander->sendAtCommand("+CIND=?");

    // Read the current indicators
    atcommander->sendAtCommand("+CIND?");

    // Activate indicator events reporting
    atcommander->sendAtCommand("+CMER=3,0,0,1");

    // Activate calling line identification
    atcommander->sendAtCommand("+CLIP=1");

    QTimer::singleShot(200, this, SLOT(reInitT68i()));
}

void PhoneController::reInitT68i()
{
    // Workaround for the T68i.
    // The one I tested didn't respond on AT+CIND=?, AT+CIND? and AT+CMER=
    // the first time, so we resend these commands after one second.
    // Otherwise the phone wont display the calling&service indicators correctly.

    atcommander->sendAtCommand("+CIND=?");
    atcommander->sendAtCommand("+CIND?");
    atcommander->sendAtCommand("+CMER=3,0,0,1");
}

// This function will map the InternalStatusInfo to
// the more abstract StatusInfo and emit a statusUpdate
// with the old and the new State;
void PhoneController::statusUpdate()
{
    StatusInfo oldState = lastExternalStatus;
    StatusInfo &newState = lastExternalStatus;
    
    // Is the socket still connected?
    newState.connected = true; //(socket && socket->isOpen());
    
    // If the phone support the call setup indicator, we'll
    // use it. Otherwise we have to guess a little.
    int callInd = state.indicatorValues["call"];
    int callSetupInd = state.indicatorValues["callsetup"];
    if (state.callsetupIndicatorSupported) {
        // We're on the safe side with the callsetup indicator
        newState.isRinging = (callSetupInd == 1);
        newState.isDialing = (callSetupInd == 2);
        newState.isAlerting = (callSetupInd == 3);
        newState.isCalling = (callInd > 0) && 
            !(newState.isRinging || newState.isDialing || newState.isAlerting);
    }
    else {
        // we have no callsetup indicator
        newState.isRinging = newState.isAlerting = false;
        newState.isCalling = newState.isDialing = false;           
        if (callInd > 0) {
            newState.isCalling = true;
        }
        else if (ringTimeoutTimer->isActive()) {
            newState.isRinging = true;
        }
    }
    newState.isIncomingCall = state.isIncomingCall;
    
    newState.canDial = !(newState.isCalling || newState.isAlerting 
        || newState.isRinging || newState.isDialing) && newState.connected;
    newState.canAccept = newState.isRinging && newState.connected;
    newState.canReject = newState.isRinging && newState.connected;
    newState.canHangup = (newState.isCalling || newState.isAlerting 
        || newState.isDialing) && newState.connected;

    newState.clipNumber = QString::null;
    newState.dialedNumber = QString::null;
    if (state.isIncomingCall) {
        if (newState.isRinging || newState.isCalling) {
            newState.clipNumber = state.clipNumber;
        }
    }
    else {
        if (newState.isAlerting || newState.isCalling) {
            newState.dialedNumber = state.dialedNumber;
        }
    }
    
    newState.hasSignalIndicator = false;
    newState.signalIndicator = 0;
    newState.peerAddress = (newState.connected ? socket->addr() : DeviceAddress::invalid);
    newState.serviceIndicator = (state.indicatorValues["service"] == 1) &&         
        newState.connected;
    newState.voiceConnection = state.voiceConnection;
               
    startStopEncoder();
    kdDebug() << "status update" << endl;

    emit statusChanged(oldState, newState);
}

void PhoneController::startStopEncoder()
{
    // Start/Stop the vorbis encoder
    KConfig* config = KGlobal::config();
    config->setGroup("recording");
    if (lastExternalStatus.isCalling && vorbisEncoder==NULL) {
        if (config->readNumEntry("recordingMode", 0) > 0) {
            QString targetPath = config->readEntry("targetPath", "/tmp");
            QDateTime now = QDateTime::currentDateTime();
            QString dateString = now.toString("yyyy-MM-dd hh:mm:ss");
            vorbisEncoder = new VorbisEncoder(this, QString("%1/%1.ogg")
                .arg(targetPath).arg(dateString));
            setVorbisTags();
            vorbisEncoder->init();
            soundServer->setVorbisEncoder(vorbisEncoder);
        }
    }
    if (lastExternalStatus.isCalling == false && vorbisEncoder != NULL) {
        QString targetFilename = vorbisEncoder->filename();
        delete vorbisEncoder;
        vorbisEncoder = NULL;
        if (config->readNumEntry("recordingMode", 0) == 2) {
            emit askSaveRecording(targetFilename);
        }
    }
}

void PhoneController::setVorbisTags()
{
    if (!vorbisEncoder) return;

    vorbisEncoder->setTag("GENRE", "phonecall");
    QDateTime ct = QDateTime::currentDateTime();
    vorbisEncoder->setTag("DATE", ct.toString("yyyy-MM-dd hh:mm:ss"));
    if (state.isIncomingCall) {
        vorbisEncoder->setTag("TITLE", QString("%1")
            .arg(state.clipNumber));
        vorbisEncoder->setTag("ALBUM", "incoming");
    }
    else {
        vorbisEncoder->setTag("TITLE", QString("%1")
            .arg(state.dialedNumber));
        vorbisEncoder->setTag("ALBUM", "outgoing");
    }
}

void PhoneController::slotUnsolicitedResult(QString s)
{
    kdDebug() << "Parsing unsolicited result: " << s << endl;
    if (parseCiev(s) == true) return;
    if (parseCindRead(s) == true) return;
    if (parseCindTest(s) == true) return;
    if (parseRing(s) == true) return;
    if (parseClip(s) == true) return;
    if (parseBvra(s) == true) return;
    kdDebug() << "Couldn't handle unsolicited result: " << s << endl;
}

bool PhoneController::parseCiev(QString s)
{
    QRegExp reg = QRegExp("^\\+CIEV: (\\d+),(\\d+)$");
    if (reg.search(s) < 0) return false;

    int index = reg.cap(1).toInt();
    int val   = reg.cap(2).toInt();
    QString indName = state.indicators[index].name;
    if (indName != QString::null) {
        state.indicatorValues[indName] = val;
        kdDebug() << QString("indicatorValues[%1] = %2")
            .arg(indName).arg(val) << endl;
        if (indName == "callsetup") {
            state.callsetupIndicatorSupported = true;
        }
    }
    statusUpdate();
    kdDebug() << "parseCiev(" << s << ") == true" << endl;
    return true;
}


bool PhoneController::parseCindRead(QString s)
{
    QRegExp reg = QRegExp("^\\+CIND: (\\d+)(,\\d+)*$");
    if (reg.search(s) < 0) return false;

    reg = QRegExp("(\\d+).*");
    int pos = 0;
    int index = 1;
    while ((pos = reg.search(s, pos)+1) > 0) {
        kdDebug() << "parseCindRead pos=" << pos << endl;
        QString indVal = reg.cap(1);
        //indIndexValueMap[index] = atoi((const char*)indVal);
        parseCiev(QString("+CIEV: %1,%2").arg(index).arg(indVal));
        ++index;
    }
    statusUpdate();
    kdDebug() << "parseCindRead(" << s << ") == true" << endl;
    return true;
}

bool PhoneController::parseCindTest(QString s)
{
    QRegExp reg = QRegExp("^\\+CIND: (.*)+$");
    if (reg.search(s) < 0) return false;

    QString indString = reg.cap(1);
    kdDebug() << "+CIND match: " << s << endl;

    if (indString[0] != '(') return false;

    reg = QRegExp("\\(\"(\\w+)\",\\(([^\\)]+)\\)\\),?");
    int pos = 0;
    int index = 1;
    while ((pos = reg.search(indString, pos)) >= 0) {
        QString indName = reg.cap(1);
        QString indValSpec = reg.cap(2);
        pos += reg.matchedLength();
        state.indicators[index].name = indName;
        kdDebug() << QString("indicators[%1].name = %2")
            .arg(index).arg(indName) << endl;
        ++index;
    }
    statusUpdate();
    kdDebug() << "parseCindTest(" << s << ") == true" << endl;
    return true;
}

bool PhoneController::parseRing(QString s)
{
    if (s.startsWith("RING")) {
        if (state.callsetupIndicatorSupported == false) {
            ringTimeoutTimer->start(5000, TRUE);
            statusUpdate();
        }
        emit ring();
        state.isIncomingCall = true;
        return true;
    }
    else {
        return false;
    }
}

void PhoneController::slotRingTimeout()
{
    statusUpdate();
}

bool PhoneController::parseClip(QString s)
{
    QRegExp reg = QRegExp("^\\+CLIP: \"([^\"]+)\".*");
    if (reg.search(s) < 0) return false;
    state.isIncomingCall = true;
    QString numberString = reg.cap(1);
    state.clipNumber = numberString;
    statusUpdate();

    return true;
}

bool PhoneController::parseBvra(QString s)
{
    QRegExp reg = QRegExp("^\\+BVRA: (\\d+)");
    if (reg.search(s) < 0) return false;
    int status = reg.cap(1).toInt();
    if (status == 0) {
        state.voiceRecognitionMode = false;
    }
    else {
        state.voiceRecognitionMode = true;
    }
    statusUpdate();

    return true;
}

void PhoneController::setVoiceRecognition(bool bOn)
{
    QString cmd = (bOn ? "+BVRA=1" : "+BVRA=0");
    if (atcommander->sendAtCommand(cmd)) {
        state.voiceRecognitionMode = bOn;
        statusUpdate();
    }
}

bool PhoneController::dial(QString number)
{
    if (status().canDial) {
        state.isIncomingCall = false;
        state.dialedNumber = number;
        atcommander->sendAtCommand(QString("D%1;").arg(number), 0, false);
        statusUpdate();
    }
    else {
        kdError() << "Tried to dial while status().canDial == false" << endl;
        return false;
    }
    return true;
}

bool PhoneController::acceptCall()
{
    bool ret;
    if (status().canAccept) {
        ret = atcommander->sendAtCommand("A", 0, false);
        statusUpdate();
    }
    else {
        kdError() << "Tried to accept while status().canAccept == false" << endl;
        ret = false;
    }
    return ret;
}

bool PhoneController::rejectCall()
{
    bool ret;
    if (status().canReject) {
        ret = atcommander->sendAtCommand("+CHUP", 0, false);
        statusUpdate();
    }
    else {
        kdError() << "Tried to reject while status().canReject == false" << endl;
        ret = false;
    }
    return ret;
}

bool PhoneController::hangupCall()
{
    bool ret;
    if (status().canHangup) {
        ret = atcommander->sendAtCommand("+CHUP", 0, false);
        statusUpdate();
    }
    else {
        kdError() << "Tried to hangup while status().canHandup == false" << endl;
        ret = false;
    }
    return ret;
}

void PhoneController::slotAudioData(Q_INT16* leftData, Q_INT16* rightData, int len)
{
    if (vorbisEncoder != NULL) {
        vorbisEncoder->writeBlock(leftData, rightData, len);
    }
}


void PhoneController::setSpeechConnection(bool bOn)
{
    kdDebug() << "PhoneController::setSpeechConnection(" << bOn << ")" << endl;
    if (bOn == false) {
        if (soundServer->connected()) {
            soundServer->closeSpeechConnection();
        }
    }
    else {
        if (soundServer->connected() == false) {
            soundServer->openSpeechConnection(socket->addr());
        }
    }
}

void PhoneController::slotSpeechConnectionStatusChanged(bool bOn)
{
    if (state.voiceConnection != bOn) {
        kdDebug() << "PhoneController::slotSpeechConnectionStatusChanged(" 
            << bOn << ")" << endl;
        state.voiceConnection = bOn;
        statusUpdate();
    }
}

const PhoneController::StatusInfo& PhoneController::status() const
{
    return lastExternalStatus;
}

void PhoneController::setSaveRecording(QString filename, bool bSave)
{
    if (!bSave) {
        QFile f(filename);
        f.remove();
    }
}

PhoneController::~PhoneController()
{
    if (socket) {
        delete socket;
    }
}


#include "phonecontroller.moc"
