/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "cardif.h"
#include "pwmexception.h"
#include "ipc.h"
#include "randomizer.h"

#include <string.h>
#include <pthread.h>
#include <time.h>
#include <chipcard.h>

/** task to execute in run() */
static cardif_task currentTask;
/** libchipcard CTCardTrader */
static CTCardTrader *ctTrader;
/** mutex for allocating/deallocating ctTrader */
static pthread_mutex_t ctTrader_mutex = PTHREAD_MUTEX_INITIALIZER;
/** libchipcard CTCard */
static CTCard *ctCard;
/** interprocess communication client */
static Ipc *threadComm_client;
/** return string from thread */
static string ret_str;
/** return unsigned 32bit from thread */
static uint32_t ret_u32;
/** the user cancelled the operation */
static bool cancelled = false;

/* IPC messages */
#define IPC_MSG_OPEN_FAILURE		"00"
#define IPC_MSG_OPEN_SUCCESS		"01"
#define IPC_MSG_CLOSE_SUCCESS		"02"
#define IPC_MSG_GEN_SIZE_TOO_SMALL	"03"
#define IPC_MSG_GEN_FAILED		"04"
#define IPC_MSG_GEN_SUCCESS		"05"
#define IPC_MSG_WRNEW_SIZE_TOO_SMALL	"06"
#define IPC_MSG_WRNEW_FAILED		"07"
#define IPC_MSG_WRNEW_SUCCESS		"08"
#define IPC_MSG_ERASE_FAILURE		"09"
#define IPC_MSG_ERASE_SUCCESS		"10"
#define IPC_MSG_GET_FAILURE		"11"
#define IPC_MSG_GET_SUCCESS		"12"
#define IPC_MSG_GET_INVALID		"13"
#define IPC_MSG_GET_WRONGVERSION	"14"
#define IPC_LINE_END			"\r\n"

/* defines for card-format */
#define CARD_MAGIC_HEADER	"PWMKEYCARD"
#define CARD_VERSION		(static_cast<char>(0x01))
#define CARD_KEY_LENGTH		(static_cast<unsigned short>(200)) /* key length: 200 bytes */
#define CARD_MINIMUM_SIZE	256			/* 256 byte == 2kBit */
#define CARD_BEGIN_OFFSET	30			/* begin at this offset */


CardIf::CardIf(unsigned int thread_stackSize)
 : CardIf_templ(thread_stackSize)
{
	threadComm_host = new Ipc;
	threadComm_client = new Ipc(threadComm_host);
	connect(threadComm_host, SIGNAL(lineAvailable(const char *, size_t)),
		this, SLOT(ipc_data(const char *, size_t)));
}

CardIf::~CardIf()
{
	delete threadComm_client;
	delete threadComm_host;
}

void CardIf::run()
{
	printDebug(string("CardIf::run() currentTask == ")
		   + tostr(static_cast<int>(currentTask)));
	switch (currentTask) {
		case task_open: {
			run_open();
			break;
		} case task_close: {
			run_close();
			break;
		} case task_genNewCard: {
			run_genNewCard();
			break;
		} case task_writeNewCard: {
			run_writeNewCard();
			break;
		} case task_getKey: {
			run_getKey();
			break;
		} case task_eraseCard: {
			run_eraseCard();
			break;
		} default: {
			BUG();
		}
	}
}

void CardIf::run_open()
{
	CTError err;
	pthread_mutex_lock(&ctTrader_mutex);
	ctTrader = new CTCardTrader(false,
				    0,
				    0,
				    CHIPCARD_STATUS_INSERTED,
				    CHIPCARD_STATUS_INSERTED |
				    CHIPCARD_STATUS_LOCKED_BY_OTHER,
				    CHIPCARD_STATUS_INSERTED);
	if (!ctTrader) {
		pthread_mutex_unlock(&ctTrader_mutex);
		const char msg[] = IPC_MSG_OPEN_FAILURE IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
		return;
	}
	pthread_mutex_unlock(&ctTrader_mutex);
	err = ctTrader->start();
	if (!err.isOk()) {
		const char msg[] = IPC_MSG_OPEN_FAILURE IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
		delete_and_null(ctTrader);
		return;
	}
	err = ctTrader->getNext(ctCard, CARDIF_OPEN_TIMEOUT);
	if (cancelled) {
		/* the user cancelled the whole thing */
		cancelled = false;
		return;
	}
	if (!err.isOk()) {
		const char msg[] = IPC_MSG_OPEN_FAILURE IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
		delete_and_null(ctTrader);
		return;
	}
	err = ctCard->openCard();
	if (!err.isOk()) {
		const char msg[] = IPC_MSG_OPEN_FAILURE IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
		delete_and_null(ctTrader);
		return;
	}
	const char msg[] = IPC_MSG_OPEN_SUCCESS IPC_LINE_END;
	threadComm_client->send(msg, array_size(msg) - 1);
}

void CardIf::run_close()
{
	run_close_nosig();
	const char msg[] = IPC_MSG_CLOSE_SUCCESS IPC_LINE_END;
	threadComm_client->send(msg, array_size(msg) - 1);
}

void CardIf::run_close_nosig()
{
	pthread_mutex_lock(&ctTrader_mutex);
	if (ctCard)
		ctCard->closeCard();
	if (ctTrader) {
		ctTrader->stop();
		delete_and_null(ctTrader);
	}
	pthread_mutex_unlock(&ctTrader_mutex);
}

void CardIf::run_genNewCard()
{
	uint32_t cardId = CardIf::genCardId();
	string key = CardIf::genKey(CARD_KEY_LENGTH);
	int ret = run_writeCard(cardId, key);
	if (ret == 0) {
		const char msg[] = IPC_MSG_GEN_SUCCESS IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
	} else if (ret == 1) {
		const char msg[] = IPC_MSG_GEN_SIZE_TOO_SMALL IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
	} else if (ret == 2) {
		const char msg[] = IPC_MSG_GEN_FAILED IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
	} else {
		BUG();
	}
}

void CardIf::run_writeNewCard()
{
	int ret = run_writeCard(ret_u32, ret_str);
	ret_u32 = 0;
	ret_str.clear();
	ret_str.reserve(0);
	if (ret == 0) {
		const char msg[] = IPC_MSG_WRNEW_SUCCESS IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
	} else if (ret == 1) {
		const char msg[] = IPC_MSG_WRNEW_SIZE_TOO_SMALL IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
	} else if (ret == 2) {
		const char msg[] = IPC_MSG_WRNEW_FAILED IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
	} else {
		BUG();
	}
}

int CardIf::run_writeCard(uint32_t cardId, const string &key)
{
	unsigned int cardSize = ctCard->memorySize();
	if (cardSize < CARD_MINIMUM_SIZE) {
		return 1;
	}
	string tmp_buf;
	// magic
	tmp_buf.append(CARD_MAGIC_HEADER, strlen(CARD_MAGIC_HEADER));
	// version number
	tmp_buf.append(CARD_VERSION, 1);
	// ID number
	char cardId_buf[sizeof(cardId)];
#ifdef BIG_ENDIAN_HOST
	cardId_buf[0] = (reinterpret_cast<char *>(&cardId))[0];
	cardId_buf[1] = (reinterpret_cast<char *>(&cardId))[1];
	cardId_buf[2] = (reinterpret_cast<char *>(&cardId))[2];
	cardId_buf[3] = (reinterpret_cast<char *>(&cardId))[3];
#else
	cardId_buf[0] = (reinterpret_cast<char *>(&cardId))[3];
	cardId_buf[1] = (reinterpret_cast<char *>(&cardId))[2];
	cardId_buf[2] = (reinterpret_cast<char *>(&cardId))[1];
	cardId_buf[3] = (reinterpret_cast<char *>(&cardId))[0];
#endif
	tmp_buf.append(cardId_buf, sizeof(cardId));
	// key length
	const unsigned short ks = CARD_KEY_LENGTH;
	char keySize_buf[sizeof(ks)];
#ifdef BIG_ENDIAN_HOST
	keySize_buf[0] = (reinterpret_cast<const char *>(&ks))[0];
	keySize_buf[1] = (reinterpret_cast<const char *>(&ks))[1];
#else
	keySize_buf[0] = (reinterpret_cast<const char *>(&ks))[1];
	keySize_buf[1] = (reinterpret_cast<const char *>(&ks))[0];
#endif
	tmp_buf.append(keySize_buf, sizeof(ks));
	// add key
	tmp_buf += key;

	CTError err = ctCard->updateBinaryRaw(tmp_buf, CARD_BEGIN_OFFSET);
	if (!err.isOk()) {
		return 2;
	}
	return 0;
}

void CardIf::run_getKey()
{
	uint32_t cardId;
	const int magicSize = strlen(CARD_MAGIC_HEADER);
	const int headerSize = magicSize			/* card-magic */
			       + sizeof(CARD_VERSION)		/* version */
			       + sizeof(cardId)			/* card-id */
			       + sizeof(CARD_KEY_LENGTH);	/* key length */
	string header_buf;
	CTError err = ctCard->readBinaryRaw(header_buf, CARD_BEGIN_OFFSET, headerSize);
	if (!err.isOk()) {
		const char msg[] = IPC_MSG_GET_FAILURE IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
		return;
	}
	int curPos = 0;
	if (header_buf.compare(curPos, magicSize, CARD_MAGIC_HEADER)) {
		const char msg[] = IPC_MSG_GET_INVALID IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
		return;
	}
	curPos += magicSize;
	if (header_buf.compare(curPos, 1, string(1, CARD_VERSION))) {
		const char msg[] = IPC_MSG_GET_WRONGVERSION IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
		return;
	}
	curPos += 1;
	char cardId_buf[sizeof(cardId)];
	memcpy(cardId_buf, header_buf.c_str() + curPos, sizeof(cardId));
	curPos += sizeof(cardId);
#ifdef BIG_ENDIAN_HOST
	(reinterpret_cast<char *>(&cardId))[0] = cardId_buf[0];
	(reinterpret_cast<char *>(&cardId))[1] = cardId_buf[1];
	(reinterpret_cast<char *>(&cardId))[2] = cardId_buf[2];
	(reinterpret_cast<char *>(&cardId))[3] = cardId_buf[3];
#else
	(reinterpret_cast<char *>(&cardId))[0] = cardId_buf[3];
	(reinterpret_cast<char *>(&cardId))[1] = cardId_buf[2];
	(reinterpret_cast<char *>(&cardId))[2] = cardId_buf[1];
	(reinterpret_cast<char *>(&cardId))[3] = cardId_buf[0];
#endif
	char keyLen_buf[sizeof(CARD_KEY_LENGTH)];
	memcpy(keyLen_buf, header_buf.c_str() + curPos, sizeof(CARD_KEY_LENGTH));
	curPos += sizeof(CARD_KEY_LENGTH);
	unsigned short keyLen;
#ifdef BIG_ENDIAN_HOST
	(reinterpret_cast<char *>(&keyLen))[0] = keyLen_buf[0];
	(reinterpret_cast<char *>(&keyLen))[1] = keyLen_buf[1];
#else
	(reinterpret_cast<char *>(&keyLen))[0] = keyLen_buf[1];
	(reinterpret_cast<char *>(&keyLen))[1] = keyLen_buf[0];
#endif
	// now read the key.
	string key_buf;
	err = ctCard->readBinaryRaw(key_buf, CARD_BEGIN_OFFSET + header_buf.size(), keyLen);
	if (!err.isOk()) {
		const char msg[] = IPC_MSG_GET_FAILURE IPC_LINE_END;
		threadComm_client->send(msg, array_size(msg) - 1);
		return;
	}

	ret_u32 = cardId;
	ret_str = key_buf;
	const char msg[] = IPC_MSG_GET_SUCCESS IPC_LINE_END;
	threadComm_client->send(msg, array_size(msg) - 1);
}

void CardIf::run_eraseCard()
{
	unsigned int cardSize, i = 0;
	cardSize = ctCard->memorySize();
	string tmp;
	tmp.assign(255, static_cast<char>(0x00));
	CTError err;
	while (i < cardSize) {
		if ((cardSize - i) < 255) {
			tmp.clear();
			tmp.assign(cardSize - i, static_cast<char>(0x00));
		}
		err = ctCard->updateBinaryRaw(tmp, 0);
		if (!err.isOk()) {
			const char msg[] = IPC_MSG_ERASE_FAILURE IPC_LINE_END;
			threadComm_client->send(msg, array_size(msg) - 1);
			return;
		}
		i += 255;
	}
	const char msg[] = IPC_MSG_ERASE_SUCCESS IPC_LINE_END;
	threadComm_client->send(msg, array_size(msg) - 1);
}

void CardIf::ipc_data(const char *buf, size_t size)
{
	if (!memcmp(buf, IPC_MSG_OPEN_FAILURE IPC_LINE_END, size)) {
		emit open_status(status_openErr);
	} else if (!memcmp(buf, IPC_MSG_OPEN_SUCCESS IPC_LINE_END, size)) {
		emit open_status(status_success);
	} else if (!memcmp(buf, IPC_MSG_GEN_SIZE_TOO_SMALL IPC_LINE_END, size)) {
		emit genNewCard_status(status_tooSmall);
	} else if (!memcmp(buf, IPC_MSG_GEN_FAILED IPC_LINE_END, size)) {
		emit genNewCard_status(status_writeErr);
	} else if (!memcmp(buf, IPC_MSG_GEN_SUCCESS IPC_LINE_END, size)) {
		emit genNewCard_status(status_success);
	} else if (!memcmp(buf, IPC_MSG_CLOSE_SUCCESS IPC_LINE_END, size)) {
		emit close_status(status_success);
	} else if (!memcmp(buf, IPC_MSG_ERASE_SUCCESS IPC_LINE_END, size)) {
		emit eraseCard_status(status_success);
	} else if (!memcmp(buf, IPC_MSG_ERASE_FAILURE IPC_LINE_END, size)) {
		emit eraseCard_status(status_failure);
	} else if (!memcmp(buf, IPC_MSG_GET_FAILURE IPC_LINE_END, size)) {
		emit getKey_status(status_failure, 0, "");
	} else if (!memcmp(buf, IPC_MSG_GET_INVALID IPC_LINE_END, size)) {
		emit getKey_status(status_invalidCard, 0, "");
	} else if (!memcmp(buf, IPC_MSG_GET_WRONGVERSION IPC_LINE_END, size)) {
		emit getKey_status(status_invalidVersion, 0, "");
	} else if (!memcmp(buf, IPC_MSG_GET_SUCCESS IPC_LINE_END, size)) {
		emit getKey_status(status_success, ret_u32, ret_str);
		ret_u32 = 0;
		ret_str.clear();
		ret_str.reserve(0);
	} else if (!memcmp(buf, IPC_MSG_WRNEW_SUCCESS IPC_LINE_END, size)) {
		emit writeNewCard_status(status_success);
	} else if (!memcmp(buf, IPC_MSG_WRNEW_SIZE_TOO_SMALL IPC_LINE_END, size)) {
		emit writeNewCard_status(status_tooSmall);
	} else if (!memcmp(buf, IPC_MSG_WRNEW_FAILED IPC_LINE_END, size)) {
		emit writeNewCard_status(status_writeErr);
	} else {
		BUG();
		printError("unhandled ipc_data message!");
	}
}

bool CardIf::open()
{
	currentTask = task_open;
	start();
	return true;
}

void CardIf::close()
{
	currentTask = task_close;
	start();
}

void CardIf::genNewCard()
{
	currentTask = task_genNewCard;
	start();
}

void CardIf::writeNewCard(uint32_t cardId, const string &key)
{
	ret_u32 = cardId;
	ret_str = key;
	currentTask = task_writeNewCard;
	start();
}

void CardIf::getKey()
{
	currentTask = task_getKey;
	start();
}

void CardIf::eraseCard()
{
	currentTask = task_eraseCard;
	start();
}

string CardIf::genKey(unsigned long size)
{
	Randomizer *rnd = Randomizer::obj();
	unsigned long i;
	string ret;
	for (i = 0; i < size; ++i) {
		ret.append(1, rnd->genRndChar());
	}
	return ret;
}

uint32_t CardIf::genCardId()
{
	Randomizer *rnd = Randomizer::obj();
	uint32_t ret;
	(reinterpret_cast<char *>(&ret))[0] = rnd->genRndChar();
	(reinterpret_cast<char *>(&ret))[1] = rnd->genRndChar();
	(reinterpret_cast<char *>(&ret))[2] = rnd->genRndChar();
	(reinterpret_cast<char *>(&ret))[3] = rnd->genRndChar();
	return ret;
}

void CardIf::cancelOp()
{
	cancelled = true;
	run_close_nosig();
}

#include "cardif.moc"
