/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "pwmkeycard.h"
#include "randomizer.h"
#include "waitwnd.h"
#include "spinforsignal.h"
#include "pwmexception.h"
#include "pwminit.h"

#include <kmessagebox.h>
#include <klocale.h>
#include <kfiledialog.h>
#include <kwin.h>


#define BACKUP_IMG_MAGIC	"PWMKEYCARD_BACKUPIMAGE"
#define BACKUP_IMG_VERSION	(0x01)


bool PwMKeyCard::ctAccess (false);

PwMKeyCard::PwMKeyCard(PwMInit *_init)
 : QObject()
{
	init = _init;
	spinner = new SpinForSignal;
	cardIf = new CardIf;
	connect(cardIf, SIGNAL(open_status(cardif_status)),
		this, SLOT(open_status(cardif_status)));
	connect(cardIf, SIGNAL(close_status(cardif_status)),
		this, SLOT(close_status(cardif_status)));
	connect(cardIf, SIGNAL(genNewCard_status(cardif_status)),
		this, SLOT(genNewCard_status(cardif_status)));
	connect(cardIf, SIGNAL(writeNewCard_status(cardif_status)),
		this, SLOT(writeNewCard_status(cardif_status)));
	connect(cardIf, SIGNAL(getKey_status(cardif_status, uint32_t, const string &)),
		this, SLOT(getKey_status(cardif_status, uint32_t, const string &)));
	connect(cardIf, SIGNAL(eraseCard_status(cardif_status)),
		this, SLOT(eraseCard_status(cardif_status)));
}

PwMKeyCard::~PwMKeyCard()
{
	delete_ifnot_null(cardIf);
	delete_ifnot_null(spinner);
}

void PwMKeyCard::open_status(cardif_status stat)
{
	removeOpenCardWaitWnd();
	switch (stat) {
		case status_success: {
			printDebug(string("PwMKeyCard::open_status() currentTask was: ")
				   + tostr(static_cast<int>(currentTask)));
			switch (currentTask) {
				case task_genNewCard: {
					cardIf->genNewCard();
					break;
				} case task_writeNewCard: {
					cardIf->writeNewCard(tmp_u32, tmp_str);
					tmp_u32 = 0;
					tmp_str.clear();
					tmp_str.reserve(0);
					break;
				} case task_getKey: {
					cardIf->getKey();
					break;
				} case task_eraseCard: {
					cardIf->eraseCard();
					break;
				} case task_close: {
				} case task_open: {
				} default: {
					BUG();
					break;
				}
			}
			rwCardWaitWnd();
			return;
		} case status_openErr: {
			KMessageBox::error(init->curWidget(),
					   i18n("Could not open chipcard;\n"
						"did you insert the correct card?"),
					   i18n("Open Error"));
			break;
		} default: {
			BUG();
			break;
		}
	}
	ctAccess = false;
	spinner->cancelSpin();
}

void PwMKeyCard::close_status(cardif_status)
{
	removeRwCardWaitWnd();
	ctAccess = false;
}

void PwMKeyCard::genNewCard_status(cardif_status stat)
{
	cardIf->close();
	switch (stat) {
		case status_success: {
			KMessageBox::information(init->curWidget(),
						 i18n("Successfully generated a new "
						      "key card. You may now use it to "
						      "encrypt your password lists."),
						 i18n("Success"));
			return;
		} case status_tooSmall: {
			KMessageBox::error(init->curWidget(),
					   i18n("This chip card is too small.\n"
						"For a PwM key card you need "
						"at least a 2kBit (256 byte) chip card."),
					   i18n("Card too Small"));
			break;
		} case status_writeErr: {
			KMessageBox::error(init->curWidget(),
					   i18n("Failed to write to the card."),
					   i18n("Write Error"));
			break;
		} default: {
			BUG();
		}
	}
}

void PwMKeyCard::writeNewCard_status(cardif_status stat)
{
	cardIf->close();
	switch (stat) {
		case status_success: {
			KMessageBox::information(init->curWidget(),
						 i18n("Successfully written the data "
						      "to the card."),
						 i18n("Write Success"));
			return;
		} case status_tooSmall: {
			KMessageBox::error(init->curWidget(),
					   i18n("This chip card is too small.\n"
						"For a PwM key card you need "
						"at least a 2kBit (256 byte) chipcard."),
					   i18n("Card too Small"));
			break;
		} case status_writeErr: {
			KMessageBox::error(init->curWidget(),
					   i18n("Failed to write to the card."),
					   i18n("Write Error"));
			break;
		} default: {
			BUG();
		}
	}
}

void PwMKeyCard::getKey_status(cardif_status stat, uint32_t cardId,
			       const string &key)
{
	cardIf->close();
	switch (stat) {
		case status_success: {
			emit keyAvailable(cardId, key);
			return;
		} case status_failure: {
			KMessageBox::error(init->curWidget(),
					   i18n("Could not read the chip card;\n"
						"is the card physically defective?"),
					   i18n("Read Error"));
			break;
		} case status_invalidCard: {
			KMessageBox::error(init->curWidget(),
					   i18n("This is not a valid PwM "
						"keycard; you can create a new "
						"one by using the \"Generate "
						"new keycard\" feature."),
					   i18n("Invalid Card"));
			break;
		} case status_invalidVersion: {
			KMessageBox::error(init->curWidget(),
					   i18n("Card version mismatch; did "
						"you generate this keycard with "
						"a older or newer version of PwM?"),
					   i18n("Invalid Version"));
			break;
		} default: {
			BUG();
		}
	}
	spinner->cancelSpin();
}

void PwMKeyCard::eraseCard_status(cardif_status stat)
{
	cardIf->close();
	switch (stat) {
		case status_success: {
			KMessageBox::information(init->curWidget(),
						 i18n("Erased card successfully."),
						 i18n("Success"));
			break;
		} case status_failure: {
			KMessageBox::error(init->curWidget(),
					   i18n("Could not erase the card."),
					   i18n("Failed"));
			break;
		} default: {
			BUG();
		}
	}
}

bool PwMKeyCard::open()
{
	if (ctAccess) {
		KMessageBox::information(init->curWidget(),
					 i18n("The card terminal is currently being read from "
					      "or written to; please stop "
					      "it first if you want to proceed."),
					 i18n("Access Failure"));
		return false;
	}
	ctAccess = true;
	if (!cardIf->open()) {
		ctAccess = false;
		return false;
	}
	openCardWaitWnd();
	return true;
}

void PwMKeyCard::genNewCard()
{
	int ret = KMessageBox::questionYesNo(init->curWidget(),
					     i18n("Do you really want to generate "
						  "a new key-card?\n"
						  "All data on the inserted card will be lost "
						  "and you will NOT be able to access "
						  "any password-list, that you encrypted "
						  "with this key-card (if you did so)."),
					     i18n("Generate Key Card?"));
	if (ret == KMessageBox::No) {
		return;
	}
	currentTask = task_genNewCard;
	if (!open()) {
		return;
	}
}

void PwMKeyCard::writeNewCard(uint32_t cardId, const string &key)
{
	tmp_u32 = cardId;
	tmp_str = key;
	currentTask = task_writeNewCard;
	open();
}

void PwMKeyCard::getKey()
{
	currentTask = task_getKey;
	if (!open()) {
		return;
	}
}

void PwMKeyCard::displayKey()
{
	connect(this, SIGNAL(keyAvailable(uint32_t, const string &)),
		this, SLOT(displayKey_slot(uint32_t, const string &)));
	getKey();
}

void PwMKeyCard::displayKey_slot(uint32_t cardId,
				   const string &)
{
	disconnect(this, SIGNAL(keyAvailable(uint32_t, const string &)),
		   this, SLOT(displayKey_slot(uint32_t, const string &)));
	char id_buf[(sizeof(cardId) * 2) + 2 /* "0x" */ + 1 /* NULL */];
	memcpy(id_buf, "0x", 2);
	sprintf(id_buf + 2, "%X", cardId);
	KMessageBox::information(init->curWidget(),
				 i18n("The identification number for "
				      "the inserted key-card is:\n")
				 + id_buf,
				 i18n("Card ID"));
}

void PwMKeyCard::eraseCard()
{
	int ret = KMessageBox::questionYesNo(init->curWidget(),
					     i18n("Do you really want to delete "
						  "the currently inserted card?\n"
						  "All data on the card will be lost "
						  "and you will NOT be able to access "
						  "any password-list, that you encrypted "
						  "with this key-card.\n"
						  "Please think before pressing "
						  "the yes-button."),
					     i18n("Erase Card?"));
	if (ret == KMessageBox::No) {
		return;
	}
	currentTask = task_eraseCard;
	if (!open()) {
		return;
	}
}

void PwMKeyCard::makeBackupImage()
{
	connect(this, SIGNAL(keyAvailable(uint32_t, const string &)),
		this, SLOT(makeBackupImage_slot(uint32_t, const string &)));
	getKey();
}

void PwMKeyCard::makeBackupImage_slot(uint32_t cardId,
				      const string &key)
{
	disconnect(this, SIGNAL(keyAvailable(uint32_t, const string &)),
		   this, SLOT(makeBackupImage_slot(uint32_t, const string &)));
	QString fn(KFileDialog::getSaveFileName(QString::null,
						i18n("*|KeyCard Backup Image"),
						init->curWidget()));
	if (fn.isEmpty())
		return;
	writeBackupImageFile(fn, key, cardId);
}

void PwMKeyCard::replayBackupImage()
{
	QString fn(KFileDialog::getOpenFileName(QString::null,
						i18n("*|KeyCard Backup Image"),
						init->curWidget()));
	if (fn.isEmpty())
		return;
	string key;
	uint32_t cardId;
	if (!readBackupImageFile(fn, &key, &cardId))
		return;
	writeNewCard(cardId, key);
}

void PwMKeyCard::writeBackupImageFile(const QString &filename, const string &key,
				      uint32_t cardId)
{
	if (QFile::exists(filename))
		QFile::remove(filename);
	QFile fd(filename);
	if (!fd.open(IO_ReadWrite)) {
		KMessageBox::error(init->curWidget(),
				   i18n("Could not open this file;\n"
					"do you have enough permissions?"),
				   i18n("Open Error"));
		return;
	}
	const char magic[] = BACKUP_IMG_MAGIC;
	int magicLen = strlen(magic);
	if (fd.writeBlock(magic, magicLen) != magicLen) {
		fd.close();
		goto writeErr;
	}
	if (fd.putch(BACKUP_IMG_VERSION) != BACKUP_IMG_VERSION) {
		fd.close();
		goto writeErr;
	}
	char cardId_buf[sizeof(cardId)];
#ifdef BIG_ENDIAN_HOST
	cardId_buf[0] = ((char *)(&cardId))[0];
	cardId_buf[1] = ((char *)(&cardId))[1];
	cardId_buf[2] = ((char *)(&cardId))[2];
	cardId_buf[3] = ((char *)(&cardId))[3];
#else
	cardId_buf[0] = ((char *)(&cardId))[3];
	cardId_buf[1] = ((char *)(&cardId))[2];
	cardId_buf[2] = ((char *)(&cardId))[1];
	cardId_buf[3] = ((char *)(&cardId))[0];
#endif
	if (fd.writeBlock(cardId_buf, sizeof(cardId_buf)) != sizeof(cardId_buf)) {
		fd.close();
		goto writeErr;
	}
	if (fd.writeBlock(key.c_str(), key.length()) != static_cast<Q_LONG>(key.length())) {
		fd.close();
		goto writeErr;
	}
	fd.close();
	KMessageBox::information(init->curWidget(),
				 i18n("Card backup was successful."),
				 i18n("Success"));
	return;
      writeErr:
	KMessageBox::error(init->curWidget(),
			   i18n("Could not write to file;\n"
				"do you have enough permissions?"),
			   i18n("Write Error"));
}

bool PwMKeyCard::readBackupImageFile(const QString &filename, string *key,
				     uint32_t *cardId)
{
	int ver, keySize;
	char cardId_buf[sizeof(*cardId)];
	char *keyBuf;
	QFile fd(filename);
	if (!fd.open(IO_ReadOnly)) {
		KMessageBox::error(init->curWidget(),
				   i18n("Could not open this file;\n"
					"do you have enough permissions?"),
				   i18n("Open Error"));
		return false;
	}
	const char magic[] = BACKUP_IMG_MAGIC;
	int magicLen = strlen(magic);
	char readMagic[sizeof(magic)];
	if (fd.readBlock(readMagic, magicLen) != magicLen) {
		fd.close();
		goto readErr;
	}
	if (memcmp(magic, readMagic, sizeof(magic))) {
		fd.close();
		goto formatErr;
	}
	ver = fd.getch();
	if (ver == -1) {
		fd.close();
		goto formatErr;
	} else if (ver != BACKUP_IMG_VERSION) {
		fd.close();
		goto versionErr;
	}
	if (fd.readBlock(cardId_buf, sizeof(cardId_buf)) != sizeof(cardId_buf)) {
		fd.close();
		goto formatErr;
	}
#ifdef BIG_ENDIAN_HOST
	((char *)(cardId))[0] = cardId_buf[0];
	((char *)(cardId))[1] = cardId_buf[1];
	((char *)(cardId))[2] = cardId_buf[2];
	((char *)(cardId))[3] = cardId_buf[3];
#else
	((char *)(cardId))[0] = cardId_buf[3];
	((char *)(cardId))[1] = cardId_buf[2];
	((char *)(cardId))[2] = cardId_buf[1];
	((char *)(cardId))[3] = cardId_buf[0];
#endif
	keySize = fd.size() - fd.at();
	if (keySize <= 0) {
		fd.close();
		goto formatErr;
	}
	keyBuf = new char[keySize];
	if (fd.readBlock(keyBuf, keySize) != keySize) {
		delete [] keyBuf;
		fd.close();
		goto formatErr;
	}
	key->assign(keyBuf, keySize);
	delete [] keyBuf;
	fd.close();
	return true;
      readErr:
	KMessageBox::error(init->curWidget(),
			   i18n("Could not read from file;\n"
				"do you have enough permissions?"),
			   i18n("Read Error"));
	return false;
      formatErr:
	KMessageBox::error(init->curWidget(),
			   i18n("This is not a valid card backup image;\n"
				"is it corrupted?"),
			   i18n("Format Error"));
	return false;
      versionErr:
	KMessageBox::error(init->curWidget(),
			   i18n("Version information of this backup image "
				"does not match; did you create the image "
				"with an older or newer version of PwM?"),
			   i18n("Version Error"));
	return false;
}

void PwMKeyCard::openCardWaitWnd()
{
	curOpenWaitWnd_timeout = CARDIF_OPEN_TIMEOUT;
	openWaitWnd_timer = new QTimer;
	connect(openWaitWnd_timer, SIGNAL(timeout()),
		this, SLOT(decOpenWaitWndTime()));
	openWaitWnd_timer->start(1000);
	openWaitWnd = new WaitWnd(i18n("PwManager: opening chipdrive..."),
				  i18n("Please insert the chipcard "
				       "into your chipdrive."),
				  true, true, init->curWidget());
	connect(openWaitWnd, SIGNAL(goingToClose()),
		this, SLOT(cancelOp()));
	openWaitWnd->show();
	KWin::setState(openWaitWnd->winId(), NET::StaysOnTop);
}

void PwMKeyCard::decOpenWaitWndTime()
{
	--curOpenWaitWnd_timeout;
	if (unlikely(curOpenWaitWnd_timeout < 0))
		curOpenWaitWnd_timeout = 0;
	openWaitWnd->updateGenericText(i18n("timeout: ")
				       + tostr(curOpenWaitWnd_timeout).c_str());
}

void PwMKeyCard::removeOpenCardWaitWnd()
{
	if (openWaitWnd_timer) {
		openWaitWnd_timer->stop();
		delete_and_null(openWaitWnd_timer);
	}
	if (openWaitWnd) {
		openWaitWnd->close();
		delete_and_null(openWaitWnd);
	}
}

void PwMKeyCard::rwCardWaitWnd()
{
	rwWaitWnd = new WaitWnd(i18n("Reading/writing chipdrive..."),
				i18n("Reading/writing the chipcard;\n"
				     "please do not remove the card."),
				false, false, init->curWidget());
	rwWaitWnd->show();
}

void PwMKeyCard::removeRwCardWaitWnd()
{
	if (rwWaitWnd) {
		rwWaitWnd->close();
		delete_and_null(rwWaitWnd);
	}
}

void PwMKeyCard::cancelOp()
{
	cardIf->cancelOp();
	ctAccess = false;
	if (openWaitWnd_timer) {
		openWaitWnd_timer->stop();
		delete_and_null(openWaitWnd_timer);
	}
	delete_ifnot_null(openWaitWnd);
	spinner->cancelSpin();
}

#include "pwmkeycard.moc"
