/***************************************************************************
 *                                                                         *
 *   copyright (C) 2003, 2004 by Michael Buesch                            *
 *   email: mbuesch@freenet.de                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License version 2        *
 *   as published by the Free Software Foundation.                         *
 *                                                                         *
 ***************************************************************************/

#include "pwmdocui.h"
#include "setmasterpwwndimpl.h"
#include "getmasterpwwndimpl.h"
#include "pwmexception.h"
#include "getkeycardwnd.h"
#include "pwm.h"
#include "globalstuff.h"
#include "spinforsignal.h"
#include "pwminit.h"

#include <qlineedit.h>
#include <qtabwidget.h>

#include <kmessagebox.h>
#include <kfiledialog.h>
#include <kwin.h>

#ifdef CONFIG_KEYCARD
# include "pwmkeycard.h"
#endif


PwMDocUi::PwMDocUi(QObject *parent, const char *name)
 : QObject(parent, name)
{
	currentView = 0;
}

PwMDocUi::~PwMDocUi()
{
}

QString PwMDocUi::requestMpw(bool chipcard)
{
	QString pw;

	if (chipcard) {
#ifdef CONFIG_KEYCARD
		uint32_t id;
		string ret;
		SpinForSignal *spinner = init->keycard()->getSpinner();
		connect(init->keycard(), SIGNAL(keyAvailable(uint32_t, const string &)),
			spinner, SLOT(u32_str_slot(uint32_t, const string &)));
		init->keycard()->getKey();
		spinner->spin(&id, &ret);
		disconnect(init->keycard(), SIGNAL(keyAvailable(uint32_t, const string &)),
			   spinner, SLOT(u32_str_slot(uint32_t, const string &)));
		if (ret.empty())
			return "";
		pw = ret.c_str();
#else // CONFIG_KEYCARD
		no_keycard_support_msg_box(currentView);
#endif // CONFIG_KEYCARD
	} else {
		GetMasterPwWndImpl pwWnd;
		KWin::setState(pwWnd.winId(), NET::StaysOnTop);
		if (pwWnd.exec() != 1)
			return "";
		pw = pwWnd.pwLineEdit->text();
	}

	return pw;
}

QString PwMDocUi::requestNewMpw(bool *chipcard)
{
	QString pw;
	SetMasterPwWndImpl pwWnd(currentView);
	pwWnd.setPwMKeyCard(init->keycard());
	if (!chipcard) {
		pwWnd.mainTab->removePage(pwWnd.mainTab->page(1));
	}
	if (pwWnd.exec() != 1)
		return "";
	pw = pwWnd.getPw(chipcard).c_str();

	return pw;
}

QString PwMDocUi::requestMpwChange(const QString *currentPw, bool *chipcard)
{
	QString pw(requestMpw(*chipcard));
	if (pw.isEmpty())
		return "";
	if (pw != *currentPw) {
		wrongMpwMsgBox(*chipcard);
		return "";
	}

	pw = requestNewMpw(chipcard);
	if (pw.isEmpty())
		return "";
	return pw;
}

void PwMDocUi::wrongMpwMsgBox(bool chipcard, QString prefix, QString postfix)
{
	QString msg;
	if (!prefix.isEmpty()) {
		msg += prefix;
		msg += "\n";
	}

	if (chipcard) {
		msg += i18n("Wrong key-card;\n"
			    "please try again with the "
			    "correct key-card.");
	} else {
		msg += i18n("Wrong master-password;\n"
			    "please try again.");
	}

	if (!postfix.isEmpty()) {
		msg += "\n";
		msg += postfix;
	}
	KMessageBox::error(currentView, msg,
			   (chipcard) ? (i18n("wrong chipcard"))
			   	      : (i18n("password error")));
}

void PwMDocUi::noMpwMsgBox(bool chipcard, QString prefix, QString postfix)
{
	QString msg;
	if (!prefix.isEmpty()) {
		msg += prefix;
		msg += "\n";
	}

	if (chipcard) {
		msg += i18n("No key-card found;\n"
			    "please insert the "
			    "correct key-card.");
	} else {
		msg += i18n("No Master Password Given.");
	}

	if (!postfix.isEmpty()) {
		msg += "\n";
		msg += postfix;
	}
	KMessageBox::error(currentView, msg,
			   (chipcard) ? (i18n("No Chipcard"))
			   	      : (i18n("Password Error")));
}

void PwMDocUi::rootAlertMsgBox()
{
	KMessageBox::error(currentView,
			   i18n("This feature is not available, "
				"if you execute PwM with \"root\" "
				"UID 0 privileges, for security reasons."),
			   i18n("Not Allowed as root."));
}

void PwMDocUi::cantDeeplock_notSavedMsgBox()
{
	KMessageBox::error(currentView,
			   i18n("Cannot deep lock, because the document "
				"has not yet been saved; please save "
				"to a file and try again."),
			   i18n("Not saved, Yet"));
}

void PwMDocUi::gpmPwLenErrMsgBox()
{
	KMessageBox::error(currentView,
			   i18n("GPasman does not support passwords "
				"shorter than 4 characters; please try "
				"again with a longer password."),
			   i18n("Password too Short"));
}

int PwMDocUi::dirtyAskSave(const QString &docTitle)
{
	int ret;
	ret = KMessageBox::questionYesNoCancel(currentView,
					       i18n("The list \"%1\" has been modified;\n"
					        "do you want to save it?").arg(docTitle),
					       i18n("Save?"));
	if (ret == KMessageBox::Yes) {
		return 0;
	} else if (ret == KMessageBox::No) {
		return 1;
	}
	// cancel
	return -1;
}

bool PwMDocUi::saveDocUi(PwMDoc *doc)
{
	PWM_ASSERT(doc);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	if (doc->isDocEmpty()) {
		KMessageBox::information(currentView,
					 i18n
					 ("Sorry, there is nothing to save;\n"
					  "please add some passwords first."),
					 i18n("Nothing to Do"));
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return true;
	}
	PwMerror ret = doc->saveDoc(conf()->confGlobCompression());
	switch (ret) {
	case e_success:
		break;
	case e_filename:
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return saveAsDocUi(doc);
	case e_weakPw:
		KMessageBox::error(currentView,
				   i18n("Error: this is a weak password;\n"
					"please select another password."),
				   i18n("Weak Password"));
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return false;
	case e_fileBackup:
		KMessageBox::error(currentView,
				   i18n("Error: could not make backup file."),
				   i18n("Backup Failed"));
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return false;
	case e_hashNotImpl:
	case e_cryptNotImpl:
		KMessageBox::error(currentView,
				   i18n("Error: Selected crypto algorithm not implemented.\n"
				        "Please make sure you have compiled PwManager with "
					"libgcrypt support."),
				   i18n("no algorithm"));
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return false;
	default:
		KMessageBox::error(currentView,
				   i18n("Error: unable to write to file;\n"
					"please check if you have permission to "
					"write to the file in that directory."),
				   i18n("Error While Writing"));
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return false;
	}
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	return true;
}

bool PwMDocUi::saveAsDocUi(PwMDoc *doc)
{
	PWM_ASSERT(doc);
	doc->timer()->getLock(DocTimer::id_autoLockTimer);
	if (doc->isDocEmpty()) {
		KMessageBox::information(currentView,
					 i18n
					 ("Sorry, there is nothing to save;\n"
					  "please add some passwords first."),
					 i18n("Nothing to Do"));
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return true;
	}
	QString fn(KFileDialog::getSaveFileName(QString::null,
						i18n("*.pwm|PwManager Password File"),
						currentView));
	if (fn.isEmpty()) {
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return false;
	}
	if (fn.right(4) != ".pwm")
		fn += ".pwm";

	PwMerror ret = doc->saveDoc(conf()->confGlobCompression(), &fn);
	switch (ret) {
	case e_success:
		break;
	case e_hashNotImpl:
	case e_cryptNotImpl:
		KMessageBox::error(currentView,
				   i18n("Error: Selected crypto algorithm not implemented.\n"
				        "Please make sure you have compiled PwManager with "
					"libgcrypt support."),
				   i18n("no algorithm"));
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return false;
	default:
		KMessageBox::error(currentView,
				   i18n("Error: unable to write to file;\n"
					"please check if you have permission to "
					"write to the file in that directory."),
				   i18n("Error While Writing"));
		doc->timer()->putLock(DocTimer::id_autoLockTimer);
		return false;
	}
	doc->timer()->putLock(DocTimer::id_autoLockTimer);
	return true;
}

bool PwMDocUi::openDocUi(PwMDoc *doc,
			 QString filename,
			 bool openDeepLocked)
{
	if (filename.isEmpty())
		filename = KFileDialog::getOpenFileName(QString::null,
							i18n("*.pwm|PwManager Password file\n"
							     "*|All files"), getCurrentView());
	if (filename.isEmpty())
		goto cancelOpen;
	PwMerror ret;
	while (true) {
		int lockStat = -1;
		if (openDeepLocked) {
			lockStat = 2;
		} else {
			if (conf()->confGlobUnlockOnOpen()) {
				lockStat = 0;
			} else {
				lockStat = 1;
			}
		}
		ret = doc->openDoc(&filename, lockStat);
		if (ret != e_success) {
			if (ret == e_readFile || ret == e_openFile) {
				KMessageBox::error(getCurrentView(),
						   i18n("Could not read file!")
						   + "\n"
						   + filename,
						   i18n("file error"));
				goto cancelOpen;
			}
			if (ret == e_alreadyOpen) {
				KMessageBox::error(getCurrentView(),
						   i18n("This file is already open."),
						   i18n("already open"));
				goto cancelOpen;
			}
			if (ret == e_fileVer) {
				KMessageBox::error(getCurrentView(),
						   i18n
						   ("File-version is not supported!\n"
						    "Did you create this file with an older or newer version of PwM?"),
						   i18n
						   ("incompatible version"));
				goto cancelOpen;
			}
			if (ret == e_wrongPw) {
				continue;
			}
			if (ret == e_noPw) {
				goto cancelOpen;
			}
			if (ret == e_fileFormat) {
				KMessageBox::error(getCurrentView(),
						   i18n
						   ("Sorry, this file has not been recognized "
						    "as a PwM Password file.\n"
						    "Probably you have selected the wrong file."),
						   i18n
						   ("no PwM password-file"));
				goto cancelOpen;
			}
			if (ret == e_fileCorrupt) {
				KMessageBox::error(getCurrentView(),
						   i18n
						   ("File corrupt!\n"
						    "Maybe the media, you stored this file on, "
						    "had bad sectors?"),
						   i18n
						   ("checksum error"));
				goto cancelOpen;
			}
		}
		break;
	}
	return true;

      cancelOpen:
	return false;
}

QString PwMDocUi::string_defaultCategory()
{
	return i18n("Default");
}

QString PwMDocUi::string_locked()
{
	return i18n("<LOCKED>");
}

QString PwMDocUi::string_deepLockedShort()
{
	return i18n("DEEP-LOCKED");
}

QString PwMDocUi::string_deepLockedLong()
{
	return i18n("This file is deep-locked:\n"
		    "that means all data has been encrypted "
		    "and written out to the file. If you want "
		    "to see the entries, please unlock the file. "
		    "While unlocking, you will be prompted for the "
		    "master-password or the key-card.");
}

QString PwMDocUi::string_defaultTitle()
{
	return i18n("Untitled");
}

#include "pwmdocui.moc"
